from table.base_table import BaseTable


class TableConfigurer:
    __slots__ = ['tables']

    tables: list[BaseTable]

    def __init__(self, **kwargs):
        self.tables = []
        if kwargs:
            for key in kwargs:
                self.tables.append(BaseTable(key, kwargs.get(key)))

    def __getattr__(self, item: str) -> BaseTable:
        return self[item]

    def __getitem__(self, item) -> BaseTable:
        if isinstance(item, int):
            return self.tables[item]
        if isinstance(item, str):
            for table in self.tables:
                if table.name == item:
                    return table
        return None

    def init_tables(self, **kwargs):
        self.tables = []
        if kwargs:
            for key in kwargs:
                self.tables.append(BaseTable(key, kwargs.get(key)))


tables: TableConfigurer = TableConfigurer()



from typing import Any

import connection_pool
from utils.enums import QueryMode


def query(sql: str, kwargs: dict, data_type: Any, model: QueryMode):
    conn: Any
    cur: Any
    try:
        conn = connection_pool.connection()
        cur = conn.cursor(connection_pool.cursor_type())
        cur.execute(sql, kwargs)

        first_data = cur.fetchone()
        if first_data is None:
            return None

        if model == QueryMode.One:
            return data_type(**first_data)
        elif model == QueryMode.Many:
            return [data_type(**first_data), *[data_type(**i) for i in cur.fetchall()]]
    finally:
        conn.close()
        cur.close()



from typing import Any, Union

from utils.conditions import Limit, Like
from utils.enums import QueryMode
from utils.query_util import query


class BaseTable:
    __slots__ = ['name', 'data_type', 'base_sql']

    name: str
    data_type: Any
    base_sql: str

    def __init__(self, name: str, data_type):
        self.name = name
        self.data_type = data_type
        self.base_sql = f'select * from {name}'

    def select_one(self, **kwargs):
        return query(self.__parse_sql(kwargs), kwargs, self.data_type, QueryMode.One)

    def select_many(self, limit: Limit = None, **kwargs) -> list:
        return query(self.__parse_sql(kwargs, limit), kwargs, self.data_type, QueryMode.Many)

    def select_like(self, likes: Union[list[Like], Like] = {}, limit: Limit = None) -> list:
        sql: str = self.base_sql
        data_dict: dict = {}
        if likes:
            sql += ' where'
            if isinstance(likes, list):
                for like in likes:
                    sql += f'{like} and'
                    data_dict[like.column] = like.value
                sql = sql.rstrip(' and')
            else:
                sql += str(likes)
                data_dict[likes.column] = likes.value
        return query(sql + str(limit) if limit else sql, data_dict, self.data_type, QueryMode.Many)

    def __parse_sql(self, conditions: dict, limit: Limit) -> str:
        sql: str = self.base_sql
        if conditions:
            sql += ' where'
            for key in conditions:
                sql += f' {key}=%({key})s and'
        sql = sql.rstrip(' and')
        return sql + str(limit) if limit else sql
